"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PgConnectionArgFilterComputedAttributesPlugin = void 0;
const utils_1 = require("./utils");
const { version } = require("../package.json");
exports.PgConnectionArgFilterComputedAttributesPlugin = {
    name: "PgConnectionArgFilterComputedAttributesPlugin",
    version,
    schema: {
        behaviorRegistry: {
            add: {
                filterBy: {
                    description: "",
                    entities: ["pgResource"],
                },
            },
        },
        entityBehavior: {
            pgResource: {
                inferred(behavior, entity, build) {
                    if (build.options.connectionFilterComputedColumns &&
                        (0, utils_1.isComputedScalarAttributeResource)(entity)) {
                        return [behavior, "filterBy"];
                    }
                    else {
                        return behavior;
                    }
                },
            },
        },
        hooks: {
            GraphQLInputObjectType_fields(inFields, build, context) {
                let fields = inFields;
                const { inflection, connectionFilterOperatorsDigest, dataplanPg: { TYPES, PgConditionStep }, EXPORTABLE, } = build;
                const { fieldWithHooks, scope: { pgCodec: codec, isPgConnectionFilter }, } = context;
                if (!isPgConnectionFilter ||
                    !codec ||
                    !codec.attributes ||
                    codec.isAnonymous) {
                    return fields;
                }
                // TODO: This may need to change once V5 fixes the need for it
                const source = Object.values(build.input.pgRegistry.pgResources).find((s) => s.codec === codec && !s.parameters && !s.isUnique);
                if (!source) {
                    return fields;
                }
                const computedAttributeResources = (0, utils_1.getComputedAttributeResources)(build, source);
                for (const computedAttributeResource of computedAttributeResources) {
                    // Must return a scalar or an array
                    if (!computedAttributeResource.isUnique) {
                        continue;
                    }
                    if (computedAttributeResource.codec.attributes) {
                        continue;
                    }
                    if (computedAttributeResource.codec === TYPES.void) {
                        continue;
                    }
                    const digest = connectionFilterOperatorsDigest(computedAttributeResource.codec);
                    if (!digest) {
                        continue;
                    }
                    const OperatorsType = build.getTypeByName(digest.operatorsTypeName);
                    if (!OperatorsType) {
                        continue;
                    }
                    if (!build.behavior.pgResourceMatches(computedAttributeResource, "filterBy")) {
                        continue;
                    }
                    const { argDetails } = build.pgGetArgDetailsFromParameters(computedAttributeResource, computedAttributeResource.parameters.slice(1));
                    // Must have only one required argument
                    if (argDetails.some((a) => a.required)) {
                        continue;
                    }
                    // Looks good
                    const fieldName = inflection.computedAttributeField({
                        resource: computedAttributeResource,
                    });
                    const functionResultCodec = computedAttributeResource.codec;
                    fields = build.extend(fields, {
                        [fieldName]: fieldWithHooks({
                            fieldName,
                            isPgConnectionFilterField: true,
                        }, {
                            description: `Filter by the object’s \`${fieldName}\` field.`,
                            type: OperatorsType,
                            applyPlan: EXPORTABLE((PgConditionStep, computedAttributeResource, functionResultCodec) => function ($where, fieldArgs) {
                                if (typeof computedAttributeResource.from !== "function") {
                                    throw new Error(`Unexpected...`);
                                }
                                const expression = computedAttributeResource.from({
                                    placeholder: $where.alias,
                                });
                                const $col = new PgConditionStep($where);
                                $col.extensions.pgFilterAttribute = {
                                    codec: functionResultCodec,
                                    expression,
                                };
                                fieldArgs.apply($col);
                            }, [
                                PgConditionStep,
                                computedAttributeResource,
                                functionResultCodec,
                            ]),
                        }),
                    }, "");
                }
                return fields;
            },
        },
    },
};
//# sourceMappingURL=PgConnectionArgFilterComputedAttributesPlugin.js.map