"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isComputedScalarAttributeResource = isComputedScalarAttributeResource;
exports.getComputedAttributeResources = getComputedAttributeResources;
exports.makeAssertAllowed = makeAssertAllowed;
function isComputedScalarAttributeResource(s) {
    if (!s.parameters || s.parameters.length < 1) {
        return false;
    }
    if (s.codec.attributes) {
        return false;
    }
    if (!s.isUnique) {
        return false;
    }
    const firstParameter = s.parameters[0];
    if (!firstParameter?.codec.attributes) {
        return false;
    }
    return true;
}
function getComputedAttributeResources(build, source) {
    const computedAttributeSources = Object.values(build.input.pgRegistry.pgResources).filter((s) => isComputedScalarAttributeResource(s) &&
        s.parameters[0].codec === source.codec);
    return computedAttributeSources;
}
// TODO: rename. (Checks that the arguments aren't null/empty.)
function makeAssertAllowed(build) {
    const { options, EXPORTABLE } = build;
    const { connectionFilterAllowNullInput, connectionFilterAllowEmptyObjectInput, } = options;
    const assertAllowed = EXPORTABLE((connectionFilterAllowEmptyObjectInput, connectionFilterAllowNullInput) => function (fieldArgs, mode) {
        const $raw = fieldArgs.getRaw();
        if (mode === "object" &&
            !connectionFilterAllowEmptyObjectInput &&
            "evalIsEmpty" in $raw &&
            $raw.evalIsEmpty()) {
            throw Object.assign(new Error("Empty objects are forbidden in filter argument input."), {
            //TODO: mark this error as safe
            });
        }
        if (mode === "list" &&
            !connectionFilterAllowEmptyObjectInput &&
            "evalLength" in $raw) {
            const l = $raw.evalLength();
            if (l != null) {
                for (let i = 0; i < l; i++) {
                    const $entry = $raw.at(i);
                    if ("evalIsEmpty" in $entry && $entry.evalIsEmpty()) {
                        throw Object.assign(new Error("Empty objects are forbidden in filter argument input."), {
                        //TODO: mark this error as safe
                        });
                    }
                }
            }
        }
        // For all modes, check null
        if (!connectionFilterAllowNullInput && $raw.evalIs(null)) {
            throw Object.assign(new Error("Null literals are forbidden in filter argument input."), {
            //TODO: mark this error as safe
            });
        }
    }, [connectionFilterAllowEmptyObjectInput, connectionFilterAllowNullInput]);
    return assertAllowed;
}
//# sourceMappingURL=utils.js.map